/* $Id: et6000_fb.c,v 1.5 1998/11/24 18:28:10 marcus Exp $
******************************************************************************

   Tseng Labs ET6000 FB operators

   Copyright (C) 1997-1998 Jim Ursetto	[jim.ursetto@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************

	This code was derived from the following sources of information:
	[1]  IBM VGA driver
	[2]  Prelease VGADOC5 by Finn Thoegersen

******************************************************************************
*/

#include <kgi/module.h>
#include <Tseng/et6000.h>
#include <Tseng/et6000_fb.h>
#include <Tseng/et6000_io.h>

/* Smooth-scrolling 24 bit probably doesn't work. */
void et6000_fb_set_origin(struct kgi_display *dpy, uint x, uint y)
{
	IS_CRITICAL;
	static uint last_xdot = -1;
	static uint last_ydot = -1;
	static uint old_offset = -1;
	register uint xpix=0,xdot=0;
	register uint ypix=0,ydot=0;
	register uint offset;
	uint fb_width=0;

	/* The chipset_command fxns have already made sure we're in bounds
	 * and have accounted for splitline. */

	switch (dpy->mode.request.graphtype) {

/* text16_set_origin handles this functionality for text mode. */
/* May have to reinsert this, though. */
/*      case KGIGT_TEXT16:
		xpix = x / dpy->mode.request.dpp.x;
		xdot = x % dpy->mode.request.dpp.x;
		ypix = y / dpy->mode.request.dpp.y;
		ydot = y % dpy->mode.request.dpp.y;
		fb_width = dpy->mode.request.virt.x * 2;
*/			
			
		case KGIGT_1BIT:
		case KGIGT_4BIT:
			xpix = x / 8;
			xdot = x % 8;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x / 8;
			break;
		case KGIGT_8BIT:
			xpix = x / 4;
			xdot = x % 4;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x;
		break;
		case KGIGT_15BIT:
		case KGIGT_16BIT:
			xpix = x / 2;
			xdot = x % 2;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x*2;
			break;
		case KGIGT_24BIT:
			xpix = (x / 4) * 3;
			xdot = x % 4;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x*3;
			break;
		case KGIGT_32BIT:
			xpix = x;
			xdot = 0;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x*4;
			break;
			
		default:
		INTERNAL_ERROR;
	}
	offset = xpix + ypix * fb_width / 4;
	
	ENTER_CRITICAL;

	if (old_offset != offset) {
		CRT_out8(offset >> 8, CRT_STARTADDR_H);
		CRT_out8(offset, CRT_STARTADDR_L);
		CRT_out8( ((offset >> 16) & CR33_DISPSTART_MASK)
		        | (CRT_in8(CRT_EXT_ADDR) &~ CR33_DISPSTART_MASK),
		        CRT_EXT_ADDR);
		old_offset = offset;
	}

	if (last_ydot != ydot) {
		CRT_out8(ydot & CR08_ROWSCAN_MASK, CRT_PRESETROW);
		last_ydot = ydot;
	}

	if (last_xdot != xdot) {

		register uint hpan;

		if (dpy->mode.request.graphtype == KGIGT_TEXT16 &&
		      dpy->mode.request.dpp.x == 9) {
			hpan = (xdot == 0) ? 0x08 : (xdot-1) & AR13_DATA_MASK;
		} else {
			hpan = (xdot & AR13_DATA_MASK) << 1;
		}

		WAIT_HVRETRACE();

/*		ATC_out8(hpan, ATC_HPAN | ATCI_ENABLEDISPLAY);*/
		ATC_out8(hpan, ATC_HPAN); 
		io_out8(ATCI_ENABLEDISPLAY, ATCI);
		last_xdot = xdot;
	}
	
	LEAVE_CRITICAL;

}

void et6000_fb_set_split(struct kgi_display *dpy, int line)
{
	register kgiu8 foo;

	enable_ext();
	CRT_out8(line & 0xFF, CRT_LINECOMPARE);
	foo = (line & 0x0100)
		? CRT_in8(0x07) | CR07_LINECOMPARE_B8
		: CRT_in8(0x07) & ~CR07_LINECOMPARE_B8;
	CRT_out8(foo, 0x07);

	foo = (line & 0x0200)
		? CRT_in8(0x09) | CR09_LINECOMPARE_B9
		: CRT_in8(0x09) & ~CR09_LINECOMPARE_B9;
	CRT_out8(foo, 0x09);

	foo = (line & 0x0400)
		? CRT_in8(CRT_OVRFLW_V) | CR35_LINECOMPARE_B10
		: CRT_in8(CRT_OVRFLW_V) & ~CR35_LINECOMPARE_B10;
	CRT_out8(foo, CRT_OVRFLW_V);
	disable_ext();
}

